/**
 * Copyright (c) 2018 - IgniteWoo.com - ALL RIGHTS RESERVED
 * 
 */

jQuery( document ).ready( function($) {

	$( '.wc-product-table' ).each( function() { 
		
		var this_table = $( this );
		
		var table_id = $( this ).attr( 'id' );

		var table = $( this ).DataTable( {
			dom: product_table_params.dom,
			serverSide: true,
			stateSave: true,
			bSort: true,
			//processing: true,
			language: product_table_params.language,
			ajax: { 
				url: product_table_params.ajax_url,
				data: function( d ) {
					d.length = product_table_params.default_per_page;
					d.nonce = product_table_params.ajax_nonce;
					d.action = 'ign_qof_load_products';
					d.table_id = table_id;
					d.columns = [];
					d.page = this_table.DataTable().page();
					ign_qof_block_form( this_table );
				},
				method: 'post',
				// Called when Ajax returns data
				dataFilter: function( data ){
					$( '.quick_order_form_outer' ).unblock();
					
					// If no data returned then remove all table rows
					try { 
						var d = $.parseJSON( data );
						if ( false == d.data || null == d.data ) { 
							this_table.find('tbody tr').remove();
						}
						
					} catch(e) { 
						//
					}
					
					return data;
				},				
			},
			drawCallback: function( settings ) {

				// Clear datatables saved state so that pagination counts work correctly
				table.state.clear();
				
				$("a[data-rel^='prettyPhoto']").prettyPhoto({
					hook: 'data-rel',
					social_tools: false,
					theme: 'pp_woocommerce',
					horizontal_padding: 20,
					opacity: 0.8,
					deeplinking: false
				});
				
				hook_form_controls( this_table ); 
				hook_qty_selector( this_table );
				
				// Disable multi-add to cart checkbox until attributes are selected
				this_table.find( '.single_variation_wrap' ).each( function() { 
					$( this ).closest( 'tr' ).find( 'td.product_checkbox input' ).attr( 'disabled', 'disabled' );
				});
						
				if ( '' !== typeof $( '#' + table_id ).DataTable().state().search.search ) { 
					try { 
						search_vars = $.parseJSON( $( '#' + table_id ).DataTable().state().search.search );
						
						$( search_vars ).each( function( k, v ) { 
							this_table.closest( '.quick_order_form_outer' ).find( ':input[name="' + v.name + '"]' ).val( v.value );
						});
						
					} catch(e) { 
						console.log( e );
					}
					
				}
			},
			//order: [ 1, 'asc' ],
			columnDefs: product_table_params.columndefs,
			responsive: {
				details: {
					type: 'column'
				},
				/*
				breakpoints: [
					{name: 'bigdesktop', width: Infinity},
					{name: 'meddesktop', width: 1480},
					{name: 'smalldesktop', width: 1280},
					{name: 'medium', width: 1188},
					{name: 'tabletl', width: 1024},
					{name: 'btwtabllandp', width: 848},
					{name: 'tabletp', width: 768},
					{name: 'mobilel', width: 480},
					{name: 'mobilep', width: 320}
				]
				*/
			},
			pageLength: product_table_params.default_per_page, // initial page length- number of records to display
			lengthMenu: product_table_params.per_page_options, //[[10, 25, 50, 100, 9999999], [10, 25, 50, 100, "All"]], //
			pagingType: "full_numbers",
		});

		table.on( 'responsive-display', function ( e, datatable, row, showHide, update ) {
			hook_form_controls( this_table );
			hook_qty_selector( this_table )
			//console.log( 'Details for row '+row.index()+' '+(showHide ? 'shown' : 'hidden') );
			//console.log( e );
		});
		
		this_table.closest( '.quick_order_form_outer' ).find( '.quick_order_product_search' ).unbind().on( 'click', function(e) { 
			e.preventDefault();
			
			if ( 'all' == $( this ).data( 'all' ) ) { 
				search_args = false; 
				this_table.closest( '.quick_order_form_outer' ).find( '.ign-quick-order-product-search' ).val('');
				this_table.closest( '.quick_order_form_outer' ).find( '#cat' ).val('');
				this_table.closest( '.quick_order_form_outer' ).find( '.quick_order_search' ).trigger( 'reset' );
				$( '#' + table_id ).DataTable().search( search_args ).draw();
				
			} else { 

				// NOTE: DT uses localstorage, which only accepts string data types
				search_args = this_table.closest( '.quick_order_form_outer' ).find( '.quick_order_search' ).serializeArray();
				search_args['table_id'] = { 'table_id': table_id }
				search_args = JSON.stringify( search_args );
				
				$( '#' + table_id ).DataTable().search( search_args ).draw().state.save();
			}
			
			return false;
		});
	});

	function hook_form_controls( this_table ) { 
		if ( 'function' == typeof qof_simple ) { 
			qof_simple();
			qof_variations();
		}
		
		this_table.find( '.single_variation_wrap' ).on( 'show_variation', function( variation ) { 
			$( this ).closest( 'tr' ).find( 'td.product_checkbox input' ).removeAttr( 'disabled' );
		});
	}

	function ign_qof_add_to_cart( btn ) { 
		var $button = $( this ),
			$cartForm = $button.closest( 'form.cart' ),
			productId = $cartForm.find( '[name="add-to-cart"]' ).val();

		// If not using AJAX, set form action to blank so current page is reloaded, rather than single product page
		if ( !table.config.ajaxCart ) {
			$cartForm.attr( 'action', '' );
			return true;
		}

		if ( typeof wc_add_to_cart_params === 'undefined' || typeof productId === 'undefined' || !$cartForm.length || $button.hasClass( 'disabled' ) ) {
			return true;
		}
		
		$button
			.removeClass( 'added' )
			.addClass( 'loading' )
			.siblings( 'a.added_to_cart' )
			.remove();
	}
	
	function ign_qof_block_form( table ) {
		$( table ).closest( '.quick_order_form_outer' ).block({
			message: null,
			overlayCSS: {
				background: '#333',
				opacity: 0.6
			},
			centerY: false,
			css: { 
				padding:        0, 
				margin:         0, 
				width:          '10%', 
				top:            '20%', 
				left:           '50%', 
				textAlign:      'center', 
				color:          '#000', 
				border:         'none',
				backgroundColor:'#fff', 
				cursor:         'wait' 
			}, 
			//message: '<div id="manager_table_processing" class="dataTables_processing card">' + product_table_params.loading + '</div>',
		});
	}

	function hook_qty_selector( table ) { 

		// Check row box if not checked when QTY selector changes
		$( table ).find( '.qty' ).unbind().on( 'click', function(e) { 
			var row = $( this ).closest( 'tr' ).find( '.product_checkbox input[type="checkbox"]' );
			var row_checked = row.is( ':checked' );
			if ( false == row_checked ) { 
				row.attr( 'checked', 'checked' );
			}
		});
		
				
		setTimeout( function() { 
			// Uncheck row box if checked and Add to Cart is clicked
			$( table ).find( '.single_add_to_cart_button' ).on( 'click', function(e) { 
				var row = $( this ).closest( 'tr' ).find( '.product_checkbox input[type="checkbox"]' );
				var row_checked = row.is( ':checked' );
				if ( true == row_checked ) { 
					row.removeAttr( 'checked', 'checked' );
				}
			});
		}, 1000 );
	}
	
	$( '.multi_atc' ).unbind().on( 'click', function(e) { 

		e.preventDefault();

		$( '.atc_message' ).html( '' );
		
		var $this_table = $( this ).closest( '.quick_order_form_outer' );
		
		var $thisbutton = $( this );
		
		$thisbutton.addClass( 'loading' );
		
		items = [];

		$this_table.find( '.wc-product-table tr' ).each( function() { 
			var tr = $( this );

			if ( tr.find( 'td.product_checkbox input' ).length > 0 ) { 
				tr.find( 'td.product_checkbox input' ).each( function() { 

					if ( $( this ).is( ':checked' ) ) {
						var val = $( this ).val();
						var qty = tr.find( 'input.qty' ).val();
						var attrs = [];
						var variation_id = false;
						
						var table_row = $( this );
						
						tr.find( '.variations_form' ).each( function() { 
							$( this ).find( 'select' ).each( function() { 
								var attr = $( this ).val();
								var name = $( this ).data( 'attribute_name' );
								attrs.push( { 'name': name, 'val' : attr } );
							});
							
							variation_id = $( this ).find( '.variation_id' ).val();
							
							single_variation = $( this ).find( '.single_variation' ).val();
						});
						
						// If single_variation is defined then the form is set to show individual variations 
						// in which case we only pass the variation ID for both parameters which then causes the 
						// PHP hook to call add_to_cart() on the product
						if ( 'undefined' !== typeof single_variation && single_variation.length >= 0 ) { 
							items.push( { 'product_id': variation_id, 'variation_id': variation_id, 'quantity' : qty, 'attrs': attrs } );
						} else { 
							items.push( { 'product_id': val, 'variation_id': variation_id, 'quantity' : qty, 'attrs': attrs } );
						}
					}
				});
			// Mobile view on small screens does not have the ".product_checkbox" table data (td) showing.
			// Parse the checkbox directly. 
			} else { 

				tr.find( '.atc_product_ids' ).each( function() { 
					if ( $( this ).is( ':checked' ) ) {

						var val = $( this ).val();
						var qty = false;
						var variation_id = false;
						var single_variation = false;
						var attrs = [];
						
						// Find parent "ul" and then location the form within it somewhere among "li" elements.
						$( this ).closest( 'ul' ).find( '.variations_form, form.cart' ).each( function() { 
							qty = $( this ).find( 'input.qty' ).val();
							variation_id = false;
							single_variation = false; 
							variation_id = $( this ).find( '.variation_id' ).val();
							single_variation = $( this ).find( '.single_variation' ).val();
						});

						// If qty is empty none were found, perhaps a different HTML layout. Try this instead:
						tr.prev( 'tr' ).find( '.variations_form:visible, form.cart:visible' ).each( function() { 
							qty = $( this ).find( 'input.qty' ).val();
							variation_id = false;
							single_variation = false; 
							variation_id = $( this ).find( '.variation_id' ).val();
							single_variation = $( this ).find( '.single_variation' ).val();
						});

						if ( 'undefined' !== typeof single_variation && single_variation.length >= 0 ) { 
							items.push( { 'product_id': variation_id, 'variation_id': variation_id, 'quantity' : qty, 'attrs': attrs } );
						} else { 
							items.push( { 'product_id': val, 'variation_id': variation_id, 'quantity' : qty, 'attrs': attrs } );
						}
					}
				});
			}
		});

		if ( items.length <= 0 ) { 
			$thisbutton.removeClass( 'loading' );
		} else if ( '0' === ign_qof_dt_params.disable_ajax_atc ) { 
			
			// Inject a form with all the data then submit the form
			var f = "<form id='non_ajax_atc' action='' method='post'><input type='hidden' name='items' value='" + JSON.stringify( items ) + "'><input type='hidden' name='action' value='ign_multi_add_to_cart'><input type='hidden' name='ign_qof_ajax_disabled' value='true'></form>";

			$( 'body' ).append( f );
			
			$( '#non_ajax_atc' ).submit();
			 
			return false;
		
		} else { 
			
			ign_qof_block_form();
			
			$.post( product_table_params.ajax_url, { action: 'ign_multi_add_to_cart', items: items }, function( response ) { 
				
				$this_table.unblock();
				$thisbutton.removeClass( 'loading' );
				
				try {
					fragments = response.fragments;
					cart_hash = response.cart_hash;

					// Block fragments class
					if ( fragments ) {
						$.each( fragments, function( key, value ) {
							$( key ).addClass( 'updating' );
							$( key ).replaceWith( value );
						});
					}
					
					$( '.atc_message' ).html( '' );
					
					if ( false !== response.success ) { 
						$( '.atc_message' ).append( response.success );
						if ( ign_add_to_cart_params.cart_redirect_after_add === 'yes' ) { 
							window.location = ign_add_to_cart_params.cart_url;
						}
					}
					
					if ( false !== response.errors ) { 
						$( '.atc_message' ).append( response.errors );
					}

					// Trigger event so themes can refresh other areas
					$( 'body' ).trigger( 'added_to_cart', [ fragments, cart_hash ] );
					$( 'body' ).trigger( 'wc_fragment_refresh', [ fragments, cart_hash ] );
					
					$this_table.find( 'td.product_checkbox input' ).each( function() { 
						$( this ).removeAttr( 'checked' );
					});
				
				} catch(e) { 
					console.log( e );
				}
			});
		}
	});
});
