<?php 
/**
 * Copyright (c) 2015 - 2017 - IgniteWoo.com - All Rights Reserved 
 
 * This is the email class to send registration email notices to admins
 */

if ( ! defined( 'ABSPATH' ) ) exit;

if ( class_exists( 'IGN_WSP_Registered_Email' ) ) { 
	return;
}

class IGN_WSP_Registered_Email extends WC_Email {
	
	public function __construct() {
		global $ignitewoo_vendors; 
		
		// set ID, this simply needs to be a unique name
		$this->id = 'ign_wsp_registered_email';
		
		// this is the title in WooCommerce Email settings
		$this->title = __( 'Wholesale Registered Email', 'ignitewoo_wholesale_pro' );
		
		// this is the description in WooCommerce email settings
		$this->description = __( "Notification sent to an admin when someone submits the wholesale registration form", 'ignitewoo_wholesale_pro' );
		
		// these are the default heading and subject lines that can be overridden using the settings
		$this->heading = __( "New User Requesting Access", 'ignitewoo_wholesale_pro' );
		$this->subject = __( "New User Requesting Access", 'ignitewoo_wholesale_pro' );
		
		$this->placeholders = array( 
			//'{first_name}' => '',
			//'{last_name}' => '', 
			'{form_fields}' => '', 
		);
		
		$this->manual = true; 
		$this->email_type = 'html';

		// Triggers
		add_action( 'ign_wsp_do_registration_email', array( $this, 'setup_and_send' ), 10, 10 );

		// Call parent constructor to load any other defaults not explicity defined here
		parent::__construct();
		
		$this->settings['email_type'] = 'html';

		// this sets the recipient to the settings defined below in init_form_fields()
		$this->recipient = '';
	}

	public function init_form_fields() {
	
		$this->form_fields = array(
			/*
			'enabled'    => array(
				'title'   => __( 'Enable', 'ignitewoo_wholesale_pro' ),
				'type'    => 'checkbox',
				'label'   => __( 'Enable this email notification', 'ignitewoo_wholesale_pro' ),
				'default' => 'yes'
			),
			*/
			'subject'    => array(
				'title'       => __( 'Email Subject', 'ignitewoo_wholesale_pro' ),
				'type'        => 'text',
				'description' => sprintf( 'This controls the email subject line. Leave blank to use the default subject: <code>%s</code>.', $this->subject ),
				'placeholder' => __( "New User Requesting Access", 'ignitewoo_wholesale_pro' ),
				'default'     => ''
			),
			'heading'    => array(
				'title'       => __( 'Email Heading', 'ignitewoo_wholesale_pro' ),
				'type'        => 'text',
				'description' => sprintf( __( 'This controls the main heading contained within the email notification. Leave blank to use the default heading: <code>%s</code>.', 'ignitewoo_wholesale_pro' ), $this->heading ),
				'placeholder' => __( "New User Requesting Access", 'ignitewoo_wholesale_pro' ),
				'default'     => ''
			),
			'message'    => array(
				'title'       => __( 'Email Message', 'ignitewoo_wholesale_pro' ),
				'type'        => 'ign_editor',
				'description' => __( 'The message body. Use <code>{form_fields}</code> to insert the data submitted in the registration form, <code>{username}</code> to insert the username  and user edit link, and <code>{email_address}</code> to insert the email address.', 'ignitewoo_wholesale_pro' ),
				'placeholder' => '',
				'default'     => __( 'User: {username}', 'ignitewoo_wholesale_pro' ) . "\n" . __( 'Email: {email_address}', 'ignitewoo_wholesale_pro' ) . "\n\n" .  __( '{form_fields}', 'ignitewoo_wholesale_pro' ),
			),
			
			// This is here to force the type to HTML, it's a hidden setting field
			'email_type' => array(
				'title'       => '',
				'type'        => 'hidden',
				'description' => '',
				'default'     => 'html',
				'class'       => 'email_type',
			)
		);
	}

	public function get_subject() {
		if ( !empty( $this->settings['subject'] ) )
			return $this->settings['subject'];
		else 
			return $this->get_default_subject();
	}

	public function get_heading() { 
		if ( !empty( $this->settings['heading'] ) )
			return $this->settings['heading'];
		else 
			return $this->get_default_heading();
	}
	
	public function get_default_subject() { 
		return $this->subject;
	}
	
	public function get_default_heading() { 
		return $this->heading;
	}
	
	public function validate_ign_editor_field( $k, $v ) {
		return $_POST[ $this->id . '_message' ];
	}
	
	public function generate_ign_editor_html( $k, $v ) {

		$content = stripslashes( isset( $this->settings['message'] ) ? $this->settings['message'] : $v['default'] );
		
		$settings = array(
			'wpautop' => true,
			'media_buttons' => false,
			'textarea_name' => $this->id . '_message',
			'teeny' => true,
			'quicktags' => false,
			'tinymce' => array( 
					'toolbar1'=> 'formatselect,bold,italic,link,undo,redo',
			)
		);
		
		ob_start();
		?>
		<tr valign="top">
			<th scope="row" class="titledesc">
				<label for="woocommerce_ign_wc_vendor_welcome_email_subject">
					<?php echo $v['title'] ?>
				</label>
			</th>
			<td class="forminp">
				<fieldset>
					<legend class="screen-reader-text"><span><?php echo $v['title'] ?> ?></span></legend>
					<p class="description"><?php echo $v['description'] ?></p>
					<?php wp_editor( $content, $this->id . '_' . $k, $settings ); ?>
					
				</fieldset>
			</td>
		</tr>
		<?php 
		return ob_get_clean();
	}
	
	public function setup_and_send( $args = array() ) {
		global $ign_wholesale_pro_suite;

		// Always send this message
		//if ( ! $this->is_enabled() )
		//	return;

		if ( !empty( $args ) ) { 
			extract( $args );
		}

		if ( empty( $id ) ) {
			return;
		}

		if ( !empty( $ign_wholesale_pro_suite->settings['ignitewoo_wsp_new_registration_to'] ) ) {
			$this->recipient = $ign_wholesale_pro_suite->settings['ignitewoo_wsp_new_registration_to'];
		} else {
			$this->recipient = get_option( 'admin_email' );
		}

		if ( empty( $this->recipient ) ) { 
			return;
		}
		
		$this->placeholders['{username}'] = sprintf( __( '<a href="%s">%s</a>', 'ignitewoo_wholesale_pro' ), admin_url( 'user-edit.php?user_id=' . $id ), $login );
		
		$this->placeholders['{email_address}'] = $email;
		
		$form_data = '';
		
		if ( !empty( $extra_data ) ) {
			ob_start();
			?>
			<table style="width:100%"> 
			<?php 
			foreach( $extra_data as $item ) { 
				?>
				<tr>
					<td valign="top" style="width:50%"><?php echo $item['title'] ?></td>
					<td valign="top" style="width:50%">
						<?php 
						if ( false !== strpos( $item['content'], "\n\n" ) ) { 
							echo wpautop( $item['content'] );
						} else {
							echo $item['content'];
						}
						?>
					</td>
				</tr>
				<?php 
			}
			?>
			</table>
			<?php 
			$form_data = ob_get_clean();
		}
		
		$this->placeholders['{form_fields}'] = $form_data; 
		
		$this->sitename = get_option( 'blogname' );
		$this->sitename = wp_specialchars_decode( $this->sitename , ENT_QUOTES );

		$this->send( $this->get_recipient(), $this->get_subject(), $this->get_content(), $this->get_headers(), $this->get_attachments() );
	}

	public function get_recipient() {
		return $this->recipient;
	}
	
	public function get_content() {
	
		$this->sending = true;
		
		// Don't let PHP output errors - which happen due to other plugins' code
		@ini_set( 'display_errors', false );
		
		$email_content = html_entity_decode( stripslashes( $this->settings['message'] ) );
		$email_content = wpautop( $email_content );
		
		foreach( $this->placeholders as $p => $v ) {
			$email_content = str_replace( $p, $v, $email_content );
		}
			
		$heading = $this->get_heading();
		
		if ( empty( $heading ) ) { 
			$heading = $this->get_default_heading();
		}
		
		ob_start();
	
		do_action( 'woocommerce_email_header', $heading, $this );
		echo $email_content;
		do_action( 'woocommerce_email_footer' );
		
		$email_content = ob_get_clean(); 
		
		return wordwrap( $email_content, 70 );
	}
	
	/*
	public function get_content_html() {
	}
	*/
	
}
